<?php

// phpcs:disable
use CRM_Loginsecurity_ExtensionUtil as E;
// phpcs:enable

class CRM_Loginsecurity_MessageTemplates {

  /**
   * Update message templates.
   * Based on CRM_Upgrade_Incremental_MessageTemplates from civicrm-core.
   */
  public static function updateTemplates() {
    // For now we do not really bother, we only have one template
    $templates = [
      [
        'name' => 'loginsecurity_access_notification',
        'title' => E::ts('Login Security - Access Notification'),
        'description' => E::ts('Notifications sent to administrators or to users, when someone accesses CiviCRM.'),
      ],
    ];

    foreach ($templates as $template) {
      // Check if the template already exists
      $workFlowID = CRM_Core_DAO::singleValueQuery("SELECT MAX(id) as id FROM civicrm_option_value WHERE name = %1", [
        1 => [$template['name'], 'String'],
      ]);

      $parts = ['html', 'text', 'subject'];
      $content = [];

      foreach ($parts as $part) {
        $content[$part] = file_get_contents(E::path('templates/message_templates/' . $template['name'] . '_' . $part . '.tpl'));
      }

      $templatesToUpdate = [];

      if ($workFlowID) {
        $id = CRM_Core_DAO::singleValueQuery("SELECT id FROM civicrm_msg_template WHERE workflow_id = %1", [
          1 => [$workFlowID, 'Positive'],
        ]);

        if ($id) {
          $templatesToUpdate[] = $id;
        }

        $defaultTemplateID = CRM_Core_DAO::singleValueQuery("
          SELECT default_template.id FROM civicrm_msg_template reserved
          LEFT JOIN civicrm_msg_template default_template
            ON reserved.workflow_id = default_template.workflow_id
          WHERE reserved.workflow_id = $workFlowID
          AND reserved.is_reserved = 1 AND default_template.is_default = 1 AND reserved.id <> default_template.id
          AND reserved.msg_html = default_template.msg_html
        ");

        if ($defaultTemplateID) {
          $templatesToUpdate[] = $defaultTemplateID;
        }

        if (!empty($templatesToUpdate)) {
          CRM_Core_DAO::executeQuery("
            UPDATE civicrm_msg_template
            SET msg_html = %1, msg_text = %2, msg_subject = %3, msg_title = %4
            WHERE id IN (" . implode(',', $templatesToUpdate) . ")", [
              1 => [$content['html'], 'String'],
              2 => [$content['text'], 'String'],
              3 => [$content['subject'], 'String'],
              4 => [$template['title'], 'String'],
            ]
          );
        }
      }
      else {
        // Create the message template
        $option_group_id = CRM_Core_DAO::singleValueQuery("SELECT id FROM civicrm_option_group WHERE name = 'msg_tpl_workflow_uf'");

        $max = CRM_Core_DAO::singleValueQuery('SELECT max(value) FROM civicrm_option_value WHERE option_group_id = %1', [
          1 => [$option_group_id, 'Positive'],
        ]);
        $max++;

        $option_value = civicrm_api3('OptionValue', 'create', [
          'description' => $template['description'],
          'is_active' => 1,
          'is_reserved' => 1,
          'label' => $template['title'],
          'name' => $template['name'],
          'option_group_id' => $option_group_id,
          'value' => $max,
          'weight' => $max,
        ]);

        // One reserved copy for the original
        \Civi\Api4\MessageTemplate::create(false)
          ->addValue('msg_title', $template['title'])
          ->addValue('msg_subject', $content['subject'])
          ->addValue('msg_text', $content['text'])
          ->addValue('msg_html', $content['html'])
          ->addValue('is_reserved', true)
          ->addValue('workflow_id', $option_value['id'])
          ->execute();

        // One non-reserved, editable by admins
        \Civi\Api4\MessageTemplate::create(false)
          ->addValue('msg_title', $template['title'])
          ->addValue('msg_subject', $content['subject'])
          ->addValue('msg_text', $content['text'])
          ->addValue('msg_html', $content['html'])
          ->addValue('is_reserved', false)
          ->addValue('workflow_id', $option_value['id'])
          ->execute();
      }
    }
  }

}
